var dhashes = [];
var exportData;
var numKeyFrames;

function handleFileSelect(target) {
    cleanup();
    document.getElementById('status').textContent = 'Loading video ...';
    loadFile(target.files[0]);
}

function loadFile(file) {
    if (file.type.match('video.*')) {
        imageReader = new FileReader();
        imageReader.readAsDataURL(file);
        imageReader.onload = (function (file) {
            return function (e) {
                renderVideo(file, e.target.result);
            };
        })(file);
    }
}

function renderVideo(file, src) {
    div = document.createElement('div');
    div.innerHTML = '<video id="video" src="' + src + '"/>';
    div.id = 'vidiv';
    document.getElementById('vids').insertBefore(div, null);

    pos = 0;
    video = document.getElementById('video');
    video.addEventListener('loadeddata', function () {
        video.currentTime = pos * 0.1;
    }, false);

    video.addEventListener('seeked', function () {
        handleVideoCapture(video, pos);
        pos++;
        if (pos <= video.duration * 10) {
            video.currentTime = pos * 0.1;
        } else {
            calcKeyFrames();
            showTable(video, file);
        }
    }, false);
}

function showTable(video, file) {
    result = '<b>Video details</b><table>';
    result += '<tr><td>Type:</td><td>&nbsp;&nbsp;&nbsp;' + file.type + '</td></tr>';
    result += '<tr><td>Size:</td><td>&nbsp;&nbsp;&nbsp;' + file.size + '</td></tr>';
    result += '<tr><td>Width:</td><td>&nbsp;&nbsp;&nbsp;' + video.videoWidth + '</td></tr>';
    result += '<tr><td>Height:</td><td>&nbsp;&nbsp;&nbsp;' + video.videoHeight + '</td></tr>';
    result += '<tr><td>Duration:</td><td>&nbsp;&nbsp;&nbsp;' + video.duration.toFixed(2) + '</td></tr>';
    result += '<tr><td>Num Key Frames:</td><td>&nbsp;&nbsp;&nbsp;' + numKeyFrames + '</td></tr>';
    result += '<tr><td>Key Frames per Second:</td><td>&nbsp;&nbsp;&nbsp;' + (numKeyFrames / video.duration).toFixed(2) + '</td></tr>';
    result += '</table>';
    span = document.createElement('span');
    span.id = 'res';
    span.innerHTML = result;
    document.getElementById('result').insertBefore(span, null);
    document.getElementById('status').textContent = 'Ready to export key frames hashes';
    document.getElementsByName('export')[0].disabled = false;
}

function handleVideoCapture(video, pos) {
    canvas = document.createElement('canvas');
    canvas.width = 8;
    canvas.height = 8;
    ctx = canvas.getContext('2d');
    ctx.drawImage(video, 0, 0, canvas.width, canvas.height);
    imageData = ctx.getImageData(0, 0, canvas.width, canvas.height);
    dhashes[pos] = dHash(imageData.data);
};

function cleanup() {
    dhashes = [];
    exportData = '';
    vidiv = document.getElementById('vidiv');
    if (vidiv) {
        vidiv.parentNode.removeChild(vidiv);
    }
    document.getElementsByName('export')[0].disabled = true;

    res = document.getElementById('res');
    if (res) {
        res.parentNode.removeChild(res);
    }
}

function calcKeyFrames() {
    numKeyFrames = 0;
    varianceTolerance = 16;
    keyHash = dhashes[0];
    exportData = '0,' + toHexHash(keyHash) + '\n';
    for (pos = 0; pos < dhashes.length; pos++) {
        if (distance(dhashes[pos], keyHash) > varianceTolerance) {
            keyHash = dhashes[pos];
            exportData += (pos * 0.1) + ',' + toHexHash(keyHash) + '\n';
            numKeyFrames++;
        }
    }
}

function handleExport() {
    anchor = document.createElement('a');
    anchor.setAttribute('href', 'data:text/csv,' + encodeURIComponent(exportData));
    anchor.setAttribute('download', 'export.csv');
    anchor.click();
}